"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _debug = _interopRequireDefault(require("debug"));

var _nodeStorage = _interopRequireDefault(require("node-storage"));

var _config = _interopRequireDefault(require("./config"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const DB_PATH = _path.default.normalize(_config.default.get("db"));

const debug = {
  info: (0, _debug.default)("storage")
};

const getStorge = function () {
  let storage = null;
  return function () {
    if (!storage) {
      storage = new _nodeStorage.default(DB_PATH);
    }

    return storage;
  };
}();

class Storage {
  constructor(db = null) {
    if (!db) {
      throw new Error('DB name can\'t be undefined');
    }

    Storage.checkDataDir();
    this._storage = getStorge();
    this._db = db;
    this._data = null;

    this._load();
  }

  _load() {
    debug.info("Load stored information");

    var data = this._storage.get(this._db);

    if (!data) {
      debug.info("There are nothing, empty");
      data = [];
    }

    this._data = data;
  }

  get length() {
    return this._data.length;
  }

  sync() {
    this._storage.put(this._db, this._data);
  }

  push(obj, sync = true) {
    this._data.push(obj);

    if (sync) {
      this.sync();
    }
  }

  remove(func, sync = true) {
    this._data = this._data.filter(x => !func(x));

    if (sync) {
      this.sync();
    }
  }

  list() {
    return this._data;
  }

  clear(sync = true) {
    debug.info(`Clear all data`);
    this._data = [];

    if (sync) {
      this.sync();
    }
  }

  each(cb) {
    return this._data.forEach(cb);
  }

  sort(cb) {
    return this._data.sort(cb);
  }

  map(cb) {
    return this._data.map(cb);
  }

  find(func) {
    debug.info(`Find`);
    return this._data.find(func);
  }

  filter(func) {
    debug.info('Filter');
    return this._data.filter(func);
  }

  reduce(func, initial = null) {
    debug.info('reduce');
    return this._data.reduce(func, initial);
  }

  static createDataDir(dirname) {
    debug.info(`Creating data directory in ${dirname}`);

    _fs.default.mkdirSync(dirname, 0o700);
  }

  static checkDataDir() {
    let stats,
        directory = _path.default.dirname(DB_PATH);

    debug.info(`Check for data directory in ${directory}`);

    try {
      stats = _fs.default.lstatSync(directory);
    } catch (e) {
      debug.info(`Don't exists`);
      return Storage.createDataDir(directory);
    }

    if (!stats.isDirectory(directory)) {
      throw new Error(`${directory} is not a directory`);
    }
  }

}

exports.default = Storage;